/*
 *  asm-i386/save_state.h
 *
 *  Copyright (C) 2001-2002 Pavel Machek <pavel@suse.cz>
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */

#ifndef _ASM_SAVE_STATE_H
#define _ASM_SAVE_STATE_H

#ifdef __KERNEL__

#include <asm/desc.h>
#include <asm/i387.h>
#include <asm/smp.h>

extern unsigned long saved_eip;
extern unsigned long saved_esp;
extern unsigned long saved_ebp;
extern unsigned long saved_ebx;
extern unsigned long saved_esi;
extern unsigned long saved_edi;

static inline void save_register_state(unsigned long return_point)
{
	saved_eip = return_point;
	asm volatile ("movl %%esp,(%0)" : "=m" (saved_esp));
	asm volatile ("movl %%ebp,(%0)" : "=m" (saved_ebp));
	asm volatile ("movl %%ebx,(%0)" : "=m" (saved_ebx));
	asm volatile ("movl %%edi,(%0)" : "=m" (saved_edi));
	asm volatile ("movl %%esi,(%0)" : "=m" (saved_esi));
}

#define restore_register_state()	do {} while (0)

#define loaddebug(thread,reg) \
               __asm__("movl %0,%%db" #reg  \
                       : /* no output */ \
                       :"r" ((thread)->debugreg[reg]))

static inline void fix_processor_context(void)
{
	int nr = smp_processor_id();
	struct tss_struct *t = &init_tss[nr];

	/* Modifies memory; clears bit in TR so that ltr will not segfault */
	set_tss_desc(nr,t);

	load_TR(nr);		/* This does ltr */
	load_LDT(current->active_mm);	/* This does lldt */

	/*
	 * Maybe reload the debug registers
	 */
	if (current->thread.debugreg[7]){
                loaddebug(&current->thread, 0);
                loaddebug(&current->thread, 1);
                loaddebug(&current->thread, 2);
                loaddebug(&current->thread, 3);
                /* no 4 and 5 */
                loaddebug(&current->thread, 6);
                loaddebug(&current->thread, 7);
	}
}

#endif /*__KERNEL__*/

#endif /*_ASM_SAVE_STATE_H*/
